/* meraki-mqtt-alpr
 *
 * meraki-mqtt-alpr is a node.js application with subscribes or consumes the MQTT notifications from second generation Cisco
 * Meraki MV cameras via an MQTT broker.  In the case of people detections it saves a still snapshot.  In the case of
 * vehicle detections (from an MV72 camera) meraki-mqtt-alpr can use a cloud based image to licence plate server - either
 * platerecognizer.com of openalpr.com.  They call these services ALPR or ANPR depending where in the world you come from.
 * If platerecognizer.com is used and you are based in New Zealand the number is then looked up in the New Zealand Police
 * stolen vehicles database to see if the vehicle is stolen.
 *
 * Either platerecognizer.com or openalpr.com can be used to convert the images to number plates in plain text.
 * I found platerecognizer substantially better, cheaper, and also comes with a lot more free lookups so you
 * can have a play without spending any money.
 * If you are based in New Zealand and are using platerecognizer.com you can also use an integration I have written
 * to do lookups against the New Zealand Police stolen vehicles database.
 *
 * If you are only interested in people detection events then you can use any second generation
 * camera such as an MV12, MV22, MV32 or MV72.
 *
 * This script assumes all cameras are in a single Meraki organisation and a single network within that organisation.  Search
 * for the word CHECKME for things you need to configure or consider.  Credentials are stored in a .env file.  Copy
 * template.env to .env and fill out the bits that are required.
 *
 * To turn this into a workable setup you'll need to install Mosquito (a free MQTT broker) on the same machine.
 * https://mosquitto.org/
 * You'll need to configure the Meraki MV72 cameras to use the MV Sense API to send their events to the MQTT broker.
 *
 * SPECIAL NOTE: Currently there is a bug in the module node-meraki-dashboard.  To work around this edit the file
 * "node_modules\node-meraki-dashboard\src\index.js" and change the "api" on line 24 from
 * https://api.meraki.com/api/v0/ to be the URL for the shard your organisation is in (which you can see when logged into
 * your Meraki dashboard with your web browser).  An example shard is "n85" - so you would change "api" to "n85".
 *
 *
 * The original project was written by Philip D'Ath (pid@ifm.net.nz) from IFM NZ Ltd (www.ifm.net.nz).
 * http://www.ifm.net.nz/cookbooks/meraki-mqtt-alpr.html
 *
 *
 * The NPM modules that were used in this project were:
 * npm install mqtt
 * https://www.npmjs.com/package/mqtt
 *
 * npm install node-meraki-dashboard
 * https://www.npmjs.com/package/node-meraki-dashboard
 *
 * npm install form-data
 * https://www.npmjs.com/package/form-data
 *
 * npm install nzpsvdb
 * https://www.npmjs.com/package/nzpsvdb
 *
 * npm install dotnet
 * https://www.npmjs.com/package/dotnet
 */

'use strict';

// Load in the credentials from .env - you can copy template.env to .env to get started.
require('dotenv').config()
if(!(process.env.merakiAPIKey && process.env.orgName && process.env.networkName))
throw("The .env file is missing or incomplete.  You can copy template.env to .env to get started.  You must fill in the three Meraki configuration values.");

// Set to true to enable console logging debugging
const debug=false

// CHECKME: Set to true to check the NZ Police stolen vehicle database
const useNZStolen=true;

// CHECKME: Set to true to use platerecognizer.com
const usePR=true

// CHECKME: Set to true to use openalpr.com
const useOpenALPR=true

// CHECKME: Set to true to save images in vehicles/ and people/ subdirectories.  Make sure you create a scheduled task
// to clean these out every so often.
const useSaveImages=true;

// Initialise the Meraki dashboard environment
const dashboard = require('node-meraki-dashboard')(process.env.merakiAPIKey);

// Used for processing images
const axios = require('axios');
const fs = require('fs');

// Connect to the MQTT broker, but we don't enable message processing till later on
const mqtt = require('mqtt')
const client = mqtt.connect('mqtt://127.0.0.1')

// Used to send images to platerecognizer.com
const FormData = require('form-data');

// Used to check the NZ Police stolen vehicles database
const nzpsvdb = require('nzpsvdb');

var orgID;
var netID;
var people = {}; // The last time stamp for each camera we have received a person count message for
var vehicles = {}; // The last time stamp for each camera we have received a vehicle count message for

// This function does nothing
const noop = () => {};

// Ask platerecognizer.com what the plate is
const processPlateRecognizer = function(cameraSerial,ts,image) {
	if(debug) console.log("processPlateRecognizer:");

	let form = new FormData();

	form.append("upload",image,{
		filename: 'vehicle.jpg'
	});
	form.append("camera_id",cameraSerial);
	form.append("timestamp",ts);

	axios.request({
		method: "post",
		url: 'https://api.platerecognizer.com/v1/plate-reader',
		data: form,
		headers: {
			'Authorization': 'Token '+process.env.plateRecognizerAPIToken,
			'Content-Type': `multipart/form-data; boundary=${form._boundary}`
		}
	}).then(function (response) {
		let results=response.data.results;
		if(results.length) {
			// We got a number plate we can read
			console.log("processPlateRecognizer: plate="+results[0].plate)

			// If enabled, check the NZ stolen plate register
			if(useNZStolen && nzpsvdb.stolen(results[0].plate))
				console.log("*** STOLEN VEHICLE DETECTED *** : "+results[0].plate);
		}
	})
	.catch(function (error) {
		console.log("processPlateRecognizer: "+error);
	});
}

// call OpenALPR and ask it what the plate is
const processOpenALPR = function(cameraSerial,ts,image) {
	if(debug) console.log("processOpenALPR:");

	axios.request({
		// CHECKME: If you are using openalpr.com then change the country code from NZ to whatever your country code is
		url: 'https://api.openalpr.com/v2/recognize_bytes?recognize_vehicle=0&country=nz&topn=1&secret_key='+process.env.openALPRsecret,
		method: "post",
		headers: {
			'Content-Type': 'image/jpeg',
		},
		data: image.toString('base64')
	}).then(function (response) {
		// Process the openalpr.com response here
		if(response.data.results.length>0) {
			// We got a number plate we can read
			console.log("processOpenALPR: plate="+response.data.results[0].plate)
		}
	})
	.catch(function (error) {
		console.log(error);
	});
}

// This gets called when a vehicle has been detected
const processVehicle = function(cameraSerial,ts,object,count) {
	if(debug) console.log("processVehicle: "+object.url+", count="+count);

	// If we can't get the image from the URL after 30 attempts then give up
	if(count>30) {
		console.log("processVehicle: could not retrieve image after 30 attempts: "+object.url);
		return;
	}

	// Download the image from the vehicle detection
	axios.request({
		url: object.url,
		method: "get",
		responseType: "arraybuffer",
		headers: {
			'Content-Type': 'image/jpeg',
		}
	}).then(function (response) {
		// If enabled, save the images
		if(useSaveImages) {
			let now=new Date().toISOString().replace(/:/, '-').replace(/:/, '-')
			fs.writeFile("vehicles/"+now+".jpg", response.data, noop);
		}

		// If enabled, use the ALPR engines (typically would only use one)
		if(useOpenALPR) processOpenALPR(cameraSerial,ts,response.data);
		if(usePR) processPlateRecognizer(cameraSerial,ts,response.data);
	})
	.catch(function (error) {
		// If a 404 got returned then we need to wait a bit longer for the image to be ready
		if(typeof error.response !=='undefined' && typeof error.response.status !== 'undefined' && error.response.status === 404) {
			setTimeout(processVehicle, 2000, cameraSerial, ts, object,count+1)
		}
		else console.log(error);
	});

}

// This gets called when a person has been detected
const processPerson = function(cameraSerial,ts,object,count) {
	if(debug) console.log("processPerson: "+object.url+", count="+count);

	// If we can't get the image from the URL after 30 attempts then give up
	if(count>30) {
		console.log("processPerson: could not retrieve image after 30 attempts: "+object.url);
		return;
	}

	// Download the image from the person detection
	axios.request({
		url: object.url,
		method: "get",
		responseType: "arraybuffer",
		headers: {
			'Content-Type': 'image/jpeg',
		}
	}).then(function (response) {
		// If enabled, save the images
		if(useSaveImages) {
			let now=new Date().toISOString().replace(/:/, '-').replace(/:/, '-')
			fs.writeFile("people/"+now+".jpg", response.data, noop);
		}

		// You can do further person processing here
	})
	.catch(function (error) {
		// If a 404 got returned then we need to wait a bit longer for the image to be ready
		if(typeof error.response !=='undefined' && typeof error.response.status !== 'undefined' && error.response.status === 404) {
			setTimeout(processPerson, 2000, cameraSerial, ts, object,count+1)
		}
		else console.log(error);
	});
}

// Request a snapshot and keep retrying until until it is ready
const processSnapshot = function(netID, cameraSerial,ts,process,count) {
	if(debug) console.log("processSnapshot: netID="+netID+", camera serial="+cameraSerial+",ts="+ts+", count="+count);

	// If we can't get the snapshot API for the URL after 10 attempts then give up
	if(count>10) {
		console.log("processSnapshot: could not process snapshot after 10 attempts");
		return;
	}

	dashboard.cameras.snapshot(netID, cameraSerial,ts)
		.then(data => setTimeout(process,5000,cameraSerial,ts,data,0))
		.catch(error => setTimeout(processSnapshot,Math.random()*30000+1000,netID,cameraSerial,ts,process,count+1));
}

// Connect to the MQTT broker and say what we are interested in
const mqttConnect = function() {
	// CHECKME: Add a subscribe for each camera (less load)
	client.subscribe('/merakimv/Q2GV-****-****/0')
	client.subscribe('/merakimv/Q2JV-****-****/0')

	// Or subscribe to all camera events if there are a small number of cameras
	// client.subscribe('/merakimv/#')
}


/* This gets called when there is a message for us to process.
 *
 * Sample topic and message
 * /merakimv/Q2GV-xxxx-xxxx/0 {"ts":1572567835359, "counts":{"person":0}}
 * /merakimv/Q2JV-xxxx-xxxx/0 {"ts":1572567835571, "counts":{"vehicle":0}}
 */
const mqttMessage = function(topic, message) {
 	if (debug) console.log("mqttMessage: "+topic + " " + message)

 	var topicArray=topic.split("/");

	// Only process meraki mv camera events that have a topic we are expecting
	if(topicArray.length !== 4) return;
	if(topicArray[1] !== "merakimv") return;
	if(topicArray[3] !== "0") return;

	// Camera serial number
	var cameraSerial=topicArray[2];

	// The message is in json format
	var data=JSON.parse(message);
	var ts=data.ts;	// The time stamp for when this event happened

	if (debug) console.log("mqttMessage: netID="+netID+", Camera serial="+cameraSerial+", timestamp="+ts);

	if(typeof data.counts.person !== 'undefined' && data.counts.person>0) {
		return;

		// A person was walked into frame
		if (debug) console.log("mqttMessage: people count: "+data.counts.person);

		// See if we have had a message from this camera in the last 500ms
		{
			let lastts=people[cameraSerial];

			if (lastts && ts<lastts+500) return;
			people[cameraSerial]=ts;
		}

//		setTimeout(processSnapshot, 60000, netID, cameraSerial,new Date(ts).toISOString(),processPerson,0)
		processSnapshot(netID, cameraSerial,new Date(ts).toISOString(),processPerson,0);
  	}
	if(typeof data.counts.vehicle !== 'undefined' && data.counts.vehicle>0) {
		// A vehicle was driven into frame
		if (debug) console.log("vehicle count: "+data.counts.vehicle);

		// See if we have had a message from this camera in the last 500ms
		{
			let lastts=vehicles[cameraSerial];

			if (lastts && ts<lastts+500) return;
			vehicles[cameraSerial]=ts;
		}

//		setTimeout(processVehicle, 60000, netID, cameraSerial,new Date(ts).toISOString(),processVehicle,0)
		processSnapshot(netID, cameraSerial,new Date(ts).toISOString(),processVehicle,0);
	}
}

// This searchs a list of Meraki networks for the ID matching the network name globally defined
const findNet = function(netList) {
	for(var i = 0; i < netList.length; i++) {
		if (netList[i].name === process.env.networkName) {
			netID=netList[i].id;

			if (debug) console.log("netID="+netID);
			return;
		}
	}
	throw new Error("Could not find network");
}


// This searchs a list of Meraki orgsfor the ID matching the orgname globally defined
const findOrg = function (orgList) {
	for(var i = 0; i < orgList.length; i++) {
		if (orgList[i].name === process.env.orgName) {
			orgID=orgList[i].id;

			if (debug) console.log("orgID="+orgID);
			return;
		}
	}
	throw new Error("Could not find organisation");
}

const main = async function(netList) {
	try {
		client.on('connect', mqttConnect);

		// Load the NZ stolen vehicles database into memory.  Note we don't wait for this to finish.
		if(useNZStolen) nzpsvdb.load();

		findOrg(await dashboard.organizations.list());
		findNet(await dashboard.networks.list(orgID));

		console.log("main: Finished initializing, starting to process camera messages.");

		client.on('message', mqttMessage);
	} catch (error) {
		client.end();
		console.log("main: "+error);
	}

}


main();
